;;
;;  mew-os2.el
;;     --- external/internal MIME methods for OS/2, and more..
;;
;;  by  OKUNISHI Fujikazu <fuji0924@mbox.kyoto-inet.or.jp>
;;
;; Created: Dec 18, 1996
;; Revised: Sep 28, 1997


(defconst mew-os2/version "mew-os2.el v0.10")

(eval-when-compile  (require 'mew-mime))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; coding-system

;(defvar mew-cs-scan      '*sjis*dos) ;; w/ nkf or wkf
 (defvar mew-cs-scan      '*iso-2022-ss2-7*dos)
 (defvar mew-cs-virtual   mew-cs-scan)

 (if (not (boundp '*noconv*dos))
   (make-coding-system  ;; xxx mew-win32.el
    '*noconv*dos 4
    ?= "No conversion." nil
    '([1 23 47 346 13 18 250 10 18 1881 5 4421 19 10
       47 772 5892 49 58 0 0 14 772 49 22] . [1 4 14 775 22])) )
 (defvar mew-cs-noconv-eol *noconv*dos) ;; base64/gzip64


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; shell specific variables [mew-summary.el and so on ..]

;(setq mew-prog-shell os2-comspec-cmd)  ;; os2-process.el
;(setq mew-prog-shell-arg "/c")


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; for *.cmd (by NAKAGAWA Takayuki)
(if (featurep 'mule) (require 'os2-process))

(defvar mew-prog-utime "utime.cmd")
(setq mew-touch-folder-p t)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; 'mew-summary-x-face (C-cC-x)
;;  rexX-Face => http://web.kyoto-inet.or.jp/people/fuji0924/os2mew.html

;(setq mew-x-face-filter '("uncompface" "icon2xbm")) ;; rexX-Face/icon2xbm.cmd
;(setq mew-x-face-prog "d:/tool/pmview/pmview.exe")  ;; PMview, GBM/2
;(setq mew-x-face-args '("/Wpos=\(,,,,For\)"))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MIME methods

(defvar running-xemacs (string-match "XEmacs" emacs-version))

(defvar mew-mime-content-type
 (list
   (list 
    "multipart/*"
    nil
    nil
    nil
    (if (boundp 'mew-icon-multipart) mew-icon-multipart)
    )
   (list 
    "audio/basic"
    "\\.au$"
    "base64"
    '("mew-mime.cmd" ("audio/basic") t)
    (if (boundp 'mew-icon-audio) mew-icon-audio)
    )
   (list
    "image/gif"
    "\\.gif$"
    "base64"
    (if (and running-xemacs (featurep 'gif))
	'(mew-mime-image/gif () nil)
      '("mew-mime.cmd" ("image/gif") t))
    (if (boundp 'mew-icon-image) mew-icon-image)
    )
   (list
    "image/jpeg"
    "\\.jpe?g$"
    "base64"
    (if (and running-xemacs (featurep 'jpeg))
	'(mew-mime-image/jpeg () nil)
      '("mew-mime.cmd" ("image/jpeg") t))
    (if (boundp 'mew-icon-image) mew-icon-image)
    )
   (list
    "image/x-xwd"
    "\\.xwd$"
    "base64"
    '("mew-mime.cmd" ("image/x-xwd") t)
    (if (boundp 'mew-icon-image) mew-icon-image)
    )
   (list
    "image/x-xbm"
    "\\.xbm$"
    "base64"
    (if (fboundp 'bitmap-insert-xbm-buffer)  ;; bitmap-mule/bitmap.el
     '(mew-mime-image/x-xbm () nil)  ;; bitmap.el
     '("mew-mime.cmd" ("image/x-xbm") t))
    (if (boundp 'mew-icon-image) mew-icon-image)
    )
   (list
    "image/x-bmp"
    "\\.bmp$"
    "base64"
    '("mew-mime.cmd" ("image/x-bmp") t)
    (if (boundp 'mew-icon-image) mew-icon-image)
    )
   (list
    "image/.*"
    "^$"
    "base64"
    '("mew-mime.cmd" ("image/oth") t)  ;; xxx
;;  '(mew-mime-application/octet-stream () nil)
    (if (boundp 'mew-icon-image) mew-icon-image)
    )
   (list
    "video/mpeg"
    "\\.mpe?g$"
    "base64"
    '("mew-mime.cmd" ("video/mpeg") t)
    (if (boundp 'mew-icon-video) mew-icon-video)
    )
   (list
    "application/postscript"
    "\\.ps$"
    "quoted-printable"
    '("mew-mime.cmd" ("application/postscript") t)
    (if (boundp 'mew-icon-application/postscript) 
	mew-icon-application/postscript)
    )
;  (list
;   "application/x-graph"
;   "\\.xg$"
;   "quoted-printable"
;   '("mew-mime.cmd" ("application/x-graph") t)
;   (if (boundp 'mew-icon-application/octet-stream) 
;	mew-icon-application/octet-stream)
;   )
   (list
    "application/pgp-keys"
    "\\.pka$"
    nil
    '(mew-mime-pgp-keys () nil)
    (if (boundp 'mew-icon-unknown) mew-icon-unknown)
    )
   (list
    "application/octet-stream"
    "\\.tar\\.?g?z?$"
    "base64"
    '(mew-mime-application/octet-stream () nil)
    (if (boundp 'mew-icon-application/octet-stream)
	mew-icon-application/octet-stream)
    )
   (list
    "application/octet-stream"
    "\\.tgz$"
    "base64"
    '(mew-mime-application/octet-stream () nil)
    (if (boundp 'mew-icon-application/octet-stream)
	mew-icon-application/octet-stream)
    )
   (list
    "application/octet-stream"
    "\\.gz$"
    "base64"
    '(mew-mime-application/octet-stream () nil)
    (if (boundp 'mew-icon-application/octet-stream)
	mew-icon-application/octet-stream)
    )
   (list
    "application/octet-stream"
    "\\.Z$"
    "base64"
    '(mew-mime-application/octet-stream () nil)
    (if (boundp 'mew-icon-application/octet-stream)
	mew-icon-application/octet-stream)
	)
   (list
    "application/octet-stream"
    "\\.pgp$"
    "base64"
    '(mew-mime-application/octet-stream () nil)
    (if (boundp 'mew-icon-application/octet-stream)
	mew-icon-application/octet-stream)
    )
   (list
    "message/external-body" 
    "\\.ext$"
    nil
    '(mew-mime-external-body () nil)
    (if (boundp 'mew-icon-message/external-body)
	mew-icon-message/external-body)
    )
   (list
    "message/rfc822"
    "^[0-9]+$"
    nil
    '(mew-mime-message/rfc822 () nil)
    (if (boundp 'mew-icon-message/rfc822) mew-icon-message/rfc822)
    )
   (list
    "message/delivery-status"
    "^$"
    nil
    '(mew-mime-text/plain () nil)
    (if (boundp 'mew-icon-text) mew-icon-text)
    )
   (list
    "text/richtext"
    "\\.rtf$"
    nil
    '(mew-mime-text/plain () nil)
    (if (boundp 'mew-icon-text) mew-icon-text)
    )
   (list
    "text/plain"
    ".*"
    nil
    '(mew-mime-text/plain () nil)
    (if (boundp 'mew-icon-text) mew-icon-text)
    )
   (list
    "text/.*"
    "^$"
    nil
    '(mew-mime-text/plain () nil)
    (if (boundp 'mew-icon-text) mew-icon-text)
    )
   (list
    ".*" 
    "^$"
    nil
    '(mew-mime-application/octet-stream () nil)
    (if (boundp 'mew-icon-unknown) mew-icon-unknown)
    )
   )
 "(content-type filename encoding (program (options ...) async) icon)"
 )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; define OS/2 specific C-T:

 (require 'mew-os2-ct)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; internal method for image/x-xbm. -- `bitmap-mule/bitmap.el' required.
;;
(defun mew-mime-image/x-xbm (begin end &optional params)
  (require 'bitmap) ;; xxx
  (if (> end begin)
      (save-excursion
	(set-buffer (mew-buffer-message))
	(let ((buffer-read-only nil))
	  (insert-buffer-substring (mew-current-get 'cache) begin end)
	  (if mew-eof-string
	      (progn
		(goto-char (point-max)) ;; necessary?
		)
		(setq last-point (point-max))
		(bitmap-insert-xbm-buffer (mew-buffer-message))
		(delete-region (point-min) last-point)
	  ))
	(set-buffer-modified-p nil) ;; xxx
	))
  )

;;; End
 (provide 'mew-os2)
