;;;			      mew-env.el
;;;
;;;		Copyright (C) 1997  Kazuhiko Yamamoto
;;;
;;;		   This emacs lisp library conforms
;;;		GNU GENERAL PUBLIC LICENSE Version 2.
;;;
;;; Author:  Kazu Yamamoto <Kazu@Mew.org>
;;; Created: Mar  6, 1997
;;; Revised: Oct 25, 1997
;;;

(defconst mew-env-version "mew-env.el version 0.12")

(require 'mew)

(defvar mew-temacs-p nil)
(defvar mew-xemacs-p nil)
(defvar mew-mule-p   nil)
(defvar mew-icon-p   nil)
(defvar mew-mule-version 0)

(if (featurep 'mule) (setq mew-mule-p t))

(cond
 ((string-match "XEmacs" emacs-version)
  (setq mew-temacs-p nil)
  (setq mew-xemacs-p t)
  (require 'mew-xemacs)
  (if (and (featurep 'xpm) (eq (console-type) 'x))
      (setq mew-icon-p t))
  (if (string-match "^19" emacs-version)
      (require 'mew-mule0)
    (require 'mew-mule3)
    (setq mew-mule-version 3))
  )
 (t
  (setq mew-temacs-p t)
  (setq mew-xemacs-p nil)
  (require 'mew-temacs)
  (if (null mew-mule-p)
    (require 'mew-mule0)
    (if (= (string-to-int mule-version) 3)
      (progn
        (require 'mew-mule3)
        (setq mew-mule-version 3)
      )
      (require 'mew-mule2)
      (setq mew-mule-version 2)
    )
   )
  )
 )

(cond 
 (mew-xemacs-p
  (or (find-face 'underline)
      (progn (make-face 'underline)
	     (set-face-underline-p 'underline t)))
  (fset 'mew-property-p (symbol-function 'extentp))
  (fset 'mew-property-make (symbol-function 'make-extent))
  (defun mew-property-move (overlay beg end &optional buffer)
    (set-extent-endpoints overlay beg end))
  (defun mew-property-put (overlay prop value)
    (set-extent-property overlay prop value))
  (fset 'mew-buffer-substring (symbol-function 'buffer-substring))
  (defun mew-mark () (mark t))
  )
 (mew-temacs-p
  (require 'faces)
  (fset 'mew-property-p (symbol-function 'overlayp))
  (fset 'mew-property-make (symbol-function 'make-overlay))
  (fset 'mew-property-move (symbol-function 'move-overlay))
  (fset 'mew-property-put (symbol-function 'overlay-put))
  (require 'easymenu)
  (if (fboundp 'buffer-substring-no-properties)
      (fset 'mew-buffer-substring
	    (symbol-function 'buffer-substring-no-properties))
    (defun mew-buffer-substring (beg end)
      "Return the text from BEG to END, without text properties, as a string."
      (let ((string (buffer-substring beg end)))
	(set-text-properties 0 (length string) nil string)
	string)))
  (defun mew-mark () (marker-position (mark-marker)))
  )
 )

(if (fboundp 'characterp)
    (fset 'mew-characterp (symbol-function 'characterp))
  (fset 'mew-characterp (symbol-function 'integerp)))

;; to avoid competition with mh-e.el, sigh.
(if (rassq 'mh-letter-mode auto-mode-alist)
    (setq auto-mode-alist
	  (delete (rassq 'mh-letter-mode auto-mode-alist)
		  auto-mode-alist)))

(defvar mew-connection-type1 nil
  "Connection type for many processes. t means PTY and nil means PIPE.
PIPE is usually recommended for speed but some OSes such as Linux 
requires PTY.")

(defvar mew-connection-type2 t 
  "Connection type for processes that requires a password. 
This must be PTY.")

;; Some OSes don't have symbolic link, sigh.
(if (fboundp 'make-symbolic-link)
    (defalias 'mew-symbolic-link 'make-symbolic-link)
  (defun mew-symbolic-link (filename newname &optional OK-IF-ALREADY-EXISTS)
    (copy-file filename newname nil t)
    )
  )

(provide 'mew-env)
