;;;			    mew-highlight.el
;;;
;;;		Copyright (C) 1997  Kazuhiko Yamamoto
;;;
;;;		   This emacs lisp library conforms
;;;		GNU GENERAL PUBLIC LICENSE Version 2.
;;;
;;; Author:  Kazu Yamamoto <Kazu@Mew.org>
;;; Created: Oct 18, 1997
;;; Revised: Oct 25, 1997
;;;

(defconst mew-highlight-version "mew-highlight.el version 0.02")

(require 'mew)

;;
;; options
;;

(defvar mew-opt-highlight-cursor-line t
  "*Put underline on the current line in Summary mode.")
(defvar mew-opt-highlight-mouse-line mew-xemacs-p
  "*Paint the line where mouse locates in Summary mode.")
(defvar mew-opt-highlight-mark t
  "*Paint marked lines in Summary mode.")
(defvar mew-opt-highlight-header t
  "*Paint header in Message and Draft mode.")
(defvar mew-opt-highlight-url t
  "*Emphasize URL lines in Message mode.")
(defvar mew-opt-highlight-url-regex
  "\\b\\(s?https?\\|ftp\\|file\\|gopher\\|news\\|telnet\\|wais\\|mailto\\):\\(//[-a-zA-Z0-9_.]+:[0-9]*\\)?[-a-zA-Z0-9_=?#$@~`%&*+|\\/.,]*[-a-zA-Z0-9_=#$@~`%&*+|\\/]"
  "*Regular expression to find URL.")

(defvar mew-opt-highlight-x-face mew-xemacs-p
  "*Iconify X-Face: on XEmacs in Message mode.")

;;
;; Styles and colors
;;

(defvar mew-highlight-cursor-line-face 'underline
  "*Face to highlight the cursor line in Summary and Virtual mode")
(defvar mew-highlight-mouse-line-function
  (function mode-motion-highlight-line)
  "*A function to highlight the mouse line in Summary and Virtual mode")
(defvar mew-highlight-url-face         'bold
  "*Face to highlight URL in Mesasge mode")
(defvar mew-highlight-url-mouse-face   'highlight
  "*Mouse face to highlight URLin Mesasge mode")

(defvar mew-highlight-header-face-list
  '(mew-highlight-header-face-subject
    mew-highlight-header-face-from
    mew-highlight-header-face-to
    mew-highlight-header-face-key
    mew-highlight-header-face-private
    mew-highlight-header-face-important
    mew-highlight-header-face-marginal
    mew-highlight-header-face-xmew)
  "*A list of face symbol name to highlight header.
Each name should be 'mew-highlight-header-face-xxx'.
Each face will be created from 'mew-highlight-header-sytel-xxx' and
'mew-highlight-header-color-xxx'. These faces can be used in
'mew-highlight-header-keywords'.")

(defvar mew-highlight-header-style-subject   'bold)
(defvar mew-highlight-header-style-from      'bold)
(defvar mew-highlight-header-style-to        'bold)
(defvar mew-highlight-header-style-key       'bold)
(defvar mew-highlight-header-style-private   'bold)
(defvar mew-highlight-header-style-important 'bold)
(defvar mew-highlight-header-style-marginal  'bold)
(defvar mew-highlight-header-style-xmew      'bold)

(defvar mew-highlight-header-color-subject   "firebrick")
(defvar mew-highlight-header-color-from      "Purple")
(defvar mew-highlight-header-color-to        "DarkOrange2")
(defvar mew-highlight-header-color-key       "ForestGreen")
(defvar mew-highlight-header-color-private   (face-foreground 'default))
(defvar mew-highlight-header-color-important "Blue")
(defvar mew-highlight-header-color-marginal  "gray50")
(defvar mew-highlight-header-color-xmew      "Red")

;; xxx hard coding...
(defvar mew-highlight-mark-folder-list '("+inbox")
  "*A folder list to highligh marked lines.")

(defvar mew-highlight-mark-face-list
  '(mew-highlight-mark-face-review
    mew-highlight-mark-face-multi
    mew-highlight-mark-face-delete
    mew-highlight-mark-face-refile)
  "*A list of face symbol name to highlight marked lines.
Each name should be 'mew-highlight-mark-face-xxx'.
Each face will be created from 'mew-highlight-mark-sytel-xxx' and
'mew-highlight-mark-color-xxx'. These faces can be used in
'mew-highlight-mark-keywords'.")

(defvar mew-highlight-mark-style-review 'default)
(defvar mew-highlight-mark-style-multi  'default)
(defvar mew-highlight-mark-style-delete 'default)
(defvar mew-highlight-mark-style-refile 'default)

(defvar mew-highlight-mark-color-review "Blue")
(defvar mew-highlight-mark-color-multi  "Purple")
(defvar mew-highlight-mark-color-delete "firebrick")
(defvar mew-highlight-mark-color-refile "ForestGreen")

(defvar mew-highlight-header-keywords
  '(("^Subject:$"
     mew-highlight-header-face-important
     mew-highlight-header-face-subject)
    ("^From:$"
     mew-highlight-header-face-important
     mew-highlight-header-face-from)
    ("^\\(To\\|Cc\\):$"
     mew-highlight-header-face-important
     mew-highlight-header-face-to)
    ("^X-Mew:$"
     mew-highlight-header-face-important
     mew-highlight-header-face-xmew)
    ("^X-.+:$"
     mew-highlight-header-face-private
     mew-highlight-header-face-marginal)
    ("^.*:$"
     mew-highlight-header-face-key
     mew-highlight-header-face-marginal))
  "*A list of (\"header-key-regex\" face-for-key face-for-value).
This is used to highlight header.")
  
(defvar mew-highlight-mark-keywords
  (list 
   (cons mew-mark-review 'mew-highlight-mark-face-review)
   (cons mew-mark-multi  'mew-highlight-mark-face-multi)
   (cons mew-mark-delete 'mew-highlight-mark-face-delete)
   (cons mew-mark-refile 'mew-highlight-mark-face-refile)
   )
  "A list of mark-face pair to used in Summary mode where its folder name
is found in 'mew-highlight-mark-folder-list'.")

;;
;; functions
;;


(defun mew-summary-highlight-setup ()
  "A function to setup mouse line for Summary and Virtual mode."
  (if (and window-system mew-xemacs-p mew-opt-highlight-mouse-line)
      (setq mode-motion-hook mew-highlight-mouse-line-function)))

(defalias 'mew-virtual-highlight-setup 'mew-summary-highlight-setup)

(defun mew-highlight-cursor-line ()
  "A function to highlight the cursor line in Summary and Virtual mode."
  (if mew-opt-highlight-cursor-line
      (let ((buffer-read-only nil))
	(if (mew-property-p mew-property-cursor-line)
	    (mew-property-move mew-property-cursor-line
			       (save-excursion (beginning-of-line) (point))
			       (save-excursion (end-of-line) (point)))
	  (setq mew-property-cursor-line
		(mew-property-make 
		 (save-excursion (beginning-of-line) (point))
		 (save-excursion (end-of-line) (point))))
	  (mew-property-put mew-property-cursor-line 'face 
			    mew-highlight-cursor-line-face)
	  ))))

(defun mew-highlight-url ()
  "A function to highlight URL in Message mode."
  (if mew-opt-highlight-url
      (let ((url-regex mew-opt-highlight-url-regex)
	    (buffer-read-only nil))
	(save-excursion
	  (goto-char (point-min))
	  (while (re-search-forward url-regex mew-bold-url-size t)
	    (put-text-property (match-beginning 0) (match-end 0)
			       'face mew-highlight-url-face)
	    (put-text-property (match-beginning 0) (match-end 0)
			       'mouse-face mew-highlight-url-mouse-face))))))

(defun mew-highlight-mark-region (beg end)
  (interactive "r")
  (if (and window-system mew-opt-highlight-mark
	   (mew-member (buffer-name) mew-highlight-mark-folder-list))
      (let ((regex (concat mew-summary-message-regex "\\([^ ]\\)"))
	    (buffer-read-only nil) face)
      (save-excursion
	(goto-char beg)
	(while (re-search-forward regex end t)
	  (setq face (cdr (assoc (string-to-char (mew-match 2))
				 mew-highlight-mark-keywords)))
	  (if face
	      (put-text-property
	       (save-excursion (beginning-of-line) (point))
	       (save-excursion (end-of-line) (point))
	       'face face)))))))

(defun mew-highlight-mark-line (mark)
  (if (and window-system mew-opt-highlight-mark
	   (mew-member (buffer-name) mew-highlight-mark-folder-list))
      (let ((buffer-read-only nil) face)
	(setq face (cdr (assoc mark mew-highlight-mark-keywords)))
	(if face
	    (put-text-property
	     (save-excursion (beginning-of-line) (point))
	     (save-excursion (end-of-line) (point))
	     'face face)))))

(defun mew-highlight-unmark-line ()
  (if (and window-system mew-opt-highlight-mark
	   (mew-member (buffer-name) mew-highlight-mark-folder-list))
      (remove-text-properties 
       (save-excursion (beginning-of-line) (point))
       (save-excursion (end-of-line) (point))
       '(face nil))))

(defun mew-highlight-header-region (beg end)
  "A function to highligh header in Message and Draft mode."
  (interactive "r")
  (save-excursion
    (save-restriction
      (narrow-to-region beg end)
      (goto-char beg)
      (let ((keywords mew-highlight-header-keywords)
	    (buffer-read-only nil)
	    key beg1 end1 beg2 end2 assoc)
	(while (not (eobp))
	  (looking-at "^\\([^:]+:\\)[ \t]*")
	  (setq beg1 (match-beginning 1))
	  (setq end1 (match-end 1))
	  (setq key (mew-match 1))
	  (setq beg2 (match-end 0))
	  (forward-line)
	  (if (setq assoc (mew-assoc-match2 key keywords 0))
	      (progn
		(put-text-property beg1 end1 'face (nth 1 assoc))
		(while (looking-at "[ \t]+") (forward-line))
		(setq end2 (1- (point)))
		(put-text-property beg2 end2 'face (nth 2 assoc)))
	    (while (looking-at "[ \t]") (forward-line))))))))

;;
;; X-Face:
;;

(defun mew-highlight-x-face (beg end)
  "A function to display X-Face."
  (if (and mew-opt-highlight-x-face mew-opt-highlight-x-face-function)
      (funcall mew-opt-highlight-x-face-function beg end)))

(cond
 (mew-xemacs-p
  (require 'highlight-headers)
  (defvar mew-opt-highlight-x-face-function
    (function (lambda (beg end)
		(interactive)
		(if (and window-system mew-opt-highlight-x-face)
		    (save-excursion
		      (goto-char beg)
		      (let ((buffer-read-only nil) xface)
			(while (re-search-forward 
				"^X-Face: *\\(.*\\(\n[ \t].*\\)*\\)\n" end t)
			  (mew-property-put
			   (mew-property-make
			    (match-beginning 0) (match-end 0))
			   'invisible t)
			  (setq xface (highlight-headers-x-face-to-pixmap
				       (match-beginning 1)
				       (match-end 1)))
			  (save-excursion
			    (goto-char beg)
			    (if (re-search-forward "^\\(From:\\).*" end t)
				(set-extent-begin-glyph 
				 (mew-property-make (match-end 1)
						    (match-end 1))
				 xface))))))))
	      )))
 (t
  (defvar mew-opt-highlight-x-face-function nil
    "*On Text Emacs, this function is called if mew-opt-highlight-x-face
is non nil. This is a temporary solution.")
  )
 )

;;
;; Setup
;;

(defun mew-highlight-face-setup (flist)
  "A function to create faces according to FLIST.
FLIST is a list of face name symbol and its name convention is
'mew-highlight-xxx-face-yyy'. A base face is copied from
'mew-highlight-xxx-style-yyy' then a color is set from
'mew-highlight-xxx-color-yyy'."
  (if window-system
      (let (fname str style color)
	(while flist
	  (setq fname (car flist))
	  (setq flist (cdr flist))
	  (set fname fname)
	  (setq str (symbol-name fname))
	  (string-match "^\\(mew-highlight-.*-\\)face\\(-.*\\)$" str)
	  (setq style (intern-soft
		       (concat (mew-match 1 str) "style" (mew-match 2 str))))
	  (setq color (intern-soft
		       (concat (mew-match 1 str) "color" (mew-match 2 str))))
	  (copy-face (symbol-value style) fname)
	  (set-face-foreground fname (symbol-value color))))))

(mew-highlight-face-setup mew-highlight-header-face-list)
(mew-highlight-face-setup mew-highlight-mark-face-list)

(provide 'mew-highlight)
