;;;			     mew-mule0.el
;;;
;;;		Copyright (C) 1997  Kazuhiko Yamamoto
;;;
;;;		   This emacs lisp library conforms
;;;		GNU GENERAL PUBLIC LICENSE Version 2.
;;;
;;; Author:  Kazu Yamamoto <Kazu@Mew.org>
;;; Created: Mar 20, 1997
;;; Revised: Oct 25, 1997
;;;

(defconst mew-mule0-version "mew-mule0.el version 0.07")

;; In the context of Mew, 'charset' means MIME charset.
;; 'cs' means the internal representation of Emacs (was known as Mule).

;;
;; User CS definitions
;;

(defvar mew-cs-noconv       nil)
(defvar mew-cs-noconv-eol   nil)
(defvar mew-cs-autoconv     nil)
(defvar mew-cs-7bit         nil)
(defvar mew-cs-7bit-crlf    nil)

(defvar mew-cs-mime-trans   nil)
(defvar mew-cs-rfc822-trans nil)
(defvar mew-cs-draft        nil)
(defvar mew-cs-scan         nil)
(defvar mew-cs-infile       nil)
(defvar mew-cs-outfile      nil)
(defvar mew-cs-virtual      nil)

(defvar mew-mule-character-set nil)

(defvar mew-cs-for-body
  '(("us-ascii"      (0)   nil "7bit")
    ("iso-8859-1"    (0 1) nil "quoted-printable"))
  )

(defvar mew-cs-for-header
  '(("us-ascii"    (0) nil nil)
    ("iso-8859-1"  (1) nil "Q"))
  )

;;
;; Charset
;;

(defun mew-charset-guess-region (beg end)
  (interactive "r")
  "Guess minimum character set name."
  (save-excursion
    (goto-char beg)
    (if (re-search-forward "[\200-\377]" end t)
	"iso-8859-1"
      "us-ascii")
    ))

(defvar mew-charset-list nil)

(defun mew-charset-to-cs (charset)
  )

(defun mew-charset-to-cte (charset)
  (if charset
      ;; used for body
      (nth 3 (mew-assoc-case-equal charset mew-cs-for-body 0))
    ))

;;
;; CS
;;

(defun mew-find-cs-region (start end)
  (let (ret)
    (save-excursion
      (goto-char beg)
      (if (re-search-forward "[\000-\177]" end t)
	  (setq ret (cons 0 ret)))
      (goto-char beg)
      (if (re-search-forward "[\200-\377]" end t)
	  (setq ret (cons 1 ret)))
      )
    ret))

(defconst mew-cs-ascii '(0))

;; to internal
(defun mew-cs-decode-region (beg end cs)
  )

;; to extenal
(defun mew-cs-encode-region (beg end cs)
  )

;; to internal
(defun mew-cs-decode-string (str cs)
  str
  )

;; to external
(defun mew-cs-encode-string (str cs)
  str
  )

;;
;; Process environment
;;

(defun mew-set-process-cs (pro from-pro to-pro)
  )

(defmacro mew-plet (&rest body)
  (`(let ((call-process-hook nil))
      (,@ body))
   ))

(defmacro mew-piolet (input output &rest body)
  (`(let ((call-process-hook nil))
      (,@ body))
   ))

(defmacro mew-flet (&rest body)
  (`(let ()
      (,@ body))
   ))

(defmacro mew-frwlet (read write &rest body)
  (`(let ()
      (,@ body))
   ))

(provide 'mew-mule0)
